<?php
/*
Plugin Name: ByDesign Replicated Site Plugin for Sub Domains
Plugin URI: http://creativeservices.bydesign.com/plugins
Description: Allows integration with ByDesign Replicated Site API via sub domain replication.
Version: 2.5.1
Author: ByDesign Technologies
Author URI: http://www.bydesign.com
Updated: 2023/02/20
*/


// add the admin options page
add_action('admin_menu', 'bd_replicated_menu');

// add the admin settings and such
add_action('admin_init', 'plugin_admin_init');

// add the get rep info action
add_action('after_setup_theme', 'replicated_subdomain_handler');

// add the tag filter for replicated tags for menu links
add_filter('wp_nav_menu_items', 'filter_replicated_tags');

// add the tag filter for replicated tags for content
add_filter('the_content', 'filter_replicated_tags');

// Add Elementor vars
if(in_array('elementor/elementor.php', apply_filters('active_plugins', get_option('active_plugins')))) {
	add_action( 'elementor/frontend/the_content', 'filter_replicated_tags');
}

/**
 * Add plugin settings
 */
function plugin_admin_init(){
	register_setting( 'bd_replicated_options', 'bd_replicated_options', 'bd_replicated_options_validate' );
	add_settings_section('bd_replicated_main', 'Main Settings', 'bd_replicated_section_text', 'bd-replicated');
	add_settings_field('bd_replicated_user', 'Username', 'bd_replicated_setting_user', 'bd-replicated', 'bd_replicated_main');
	add_settings_field('bd_replicated_password', 'Password', 'bd_replicated_setting_password', 'bd-replicated', 'bd_replicated_main');
	add_settings_field('bd_replicated_webfolder', 'Freedom WebFolder', 'bd_replicated_setting_webfolder', 'bd-replicated', 'bd_replicated_main');
	add_settings_field('bd_replocator_slug', 'About Me Slug', 'bd_replocator_setting_slug', 'bd-replicated', 'bd_replicated_main');
	add_settings_field('bd_replocator_namealias', 'Consultant Alias', 'bd_replocator_setting_namealias', 'bd-replicated', 'bd_replicated_main');
}

/**
 * Performs input fields validation
 */
function bd_replicated_options_validate($input) {
	$newinput['bd_replicated_user'] = trim($input['bd_replicated_user']);
	$newinput['bd_replicated_password'] = trim($input['bd_replicated_password']);
	$newinput['bd_replicated_webfolder'] = trim($input['bd_replicated_webfolder']);
	$newinput['bd_replocator_slug'] = trim($input['bd_replocator_slug']);
	$newinput['bd_replocator_namealias'] = trim($input['bd_replocator_namealias']);

	return $newinput;
}

/**
 * Renders the instructions text
 */
function bd_replicated_section_text() {
	echo '<p>Enter a valid Freedom Username and Password of an account that has the appropriate Replicated Site API security rights for the given Web Service URL.</p>
	<p>This version of the plugin is used for subdomain replication only (http://rep.domain.com).  If you are looking for subfolder replication (http://domain.com/rep) please contact your implementation manager.</p>';
}

/**
 * Renders the Username field
 */
function bd_replicated_setting_user() {
	$options = get_option('bd_replicated_options');
	echo "<input id='bd_replicated_user' name='bd_replicated_options[bd_replicated_user]' size='20' type='text' value='{$options['bd_replicated_user']}' />";
}

/**
 * Renders the password field
 */
function bd_replicated_setting_password() {
	$options = get_option('bd_replicated_options');
	echo "<input id='bd_replicated_password' name='bd_replicated_options[bd_replicated_password]' size='20' type='password' value='{$options['bd_replicated_password']}' />";
}

/**
 * Renders the password field
 */
function bd_replicated_setting_webfolder() {
	$options = get_option('bd_replicated_options');
	echo "<input id='bd_replicated_webfolder' name='bd_replicated_options[bd_replicated_webfolder]' size='20' type='text' value='{$options['bd_replicated_webfolder']}' />";
}


/**
 * Renders additional fields for rep locator plugin
 */
function bd_replocator_setting_slug() {
    $options = get_option('bd_replicated_options');
    echo "<input id='bd_replocator_slug' name='bd_replicated_options[bd_replocator_slug]' size='20' type='text' value='{$options['bd_replocator_slug']}' />";
}

function bd_replocator_setting_namealias() {
    $options = get_option('bd_replicated_options');
    echo "<input id='bd_replocator_namealias' name='bd_replicated_options[bd_replocator_namealias]' size='20' type='text' value='{$options['bd_replocator_namealias']}' />";
}



/**
 * Creates the menu item within the Settings top level menu
 */
function bd_replicated_menu() {
	add_menu_page('ByDesign Replicated Options', 'ByDesign Replicated', 'manage_options', 'bd-replicated', 'bd_replicated_options');
}

/**
 * Renders the plugin options in the settings screen
 */
function bd_replicated_options() {
	if (!current_user_can('manage_options'))  {
		wp_die( __('You do not have sufficient permissions to access this page.') );
	}
?>
<div class="wrap">
	<form action="options.php" method="post">
		<?php settings_fields('bd_replicated_options'); ?>
		<?php do_settings_sections('bd-replicated'); ?>
		<p class="submit"><input type="submit" name="submit" id="submit" class="button-primary" value="Save Changes"></p>
	</form>
</div>

<?php

}

//Webservices API WSDL URL

$options = get_option('bd_replicated_options');
$webfolder = $options['bd_replicated_webfolder'];

define('API_URL', 'https://api.securefreedom.com/'.$webfolder.'/webservice/replicatedsiteapi.asmx?WSDL');
define('REPLICATED_COOKIE', 'bdreplicated_cookie');


function bdt_rep_javascript() {
    ?>
<script>
<?php
// Set JS variable 
if(isset($_COOKIE[REPLICATED_COOKIE])) {
	$repInfo = unserialize(base64_decode($_COOKIE[REPLICATED_COOKIE]));
?>
var bdt_repid = <?php echo $repInfo->RepDID; ?>; 
var callback = function(){
	var bdt_anchor = document.getElementsByTagName('a');
	Array.prototype.forEach.call(bdt_anchor, function (bdt_anchor) {
		bdt_anchor.href = bdt_anchor.href.replace('_BDT_repdid_BDT_', bdt_repid);
	});
};
<?php 
} else { 
?>
var callback = function(){
	var bdt_anchor = document.getElementsByTagName('a');
	Array.prototype.forEach.call(bdt_anchor, function (bdt_anchor) {
		bdt_anchor.href = bdt_anchor.href.replace('_BDT_repdid_BDT_', '');
	});
};
<?php } ?>

if (document.readyState === "complete" || (document.readyState !== "loading" && !document.documentElement.doScroll)) {
  callback();
} else {
  document.addEventListener("DOMContentLoaded", callback);
}
</script>
    <?php
}
add_action('wp_head', 'bdt_rep_javascript');

/**
 * Replaces the Replicated tags with the actual values stored in the options database
 * Tags in $content must have the format _BDT_TAGNAME_BDT_
 */
function filter_replicated_tags($content) {
	$repInfo = replicated_is_rep_available();

	//get rep info if there is none cached already
	if($repInfo != null) {

		$tagsArray = get_object_vars($repInfo);
		$tags = array_keys($tagsArray);

		foreach($tags as $tag) {
			if($tag != 'CustomFields') {
				if(!empty($tagsArray[$tag])) {
					$content = str_ireplace('_BDT_'.$tag.'_BDT_', $tagsArray[$tag], $content);
				}
			}
			else { //process custom fields
				foreach($tagsArray[$tag] as $customField) {
					$content = str_ireplace('_BDT_Custom-'.$customField['Name'] .'_BDT_', $customField['Value'] , $content);
				}
			}
		}

		//if there are unsupported tags for some reason, or a tag has no value

		//hide elements marked as optional (enclosed in double square braces
		$content = preg_replace('/\[\[.*_BDT_.*?_BDT_.*\]\]/', '', $content);

		//remove tags from unmatched tags
		$content = preg_replace('/_BDT_.*?_BDT_/', '', $content);

		//remove double square braces from matched optional elements
		$content = str_replace('[[', '', $content);
		$content = str_replace(']]', '', $content);
	}
    else {
		//use fallback text when info is not available
		//if there are unsupported tags for some reason, or a tag has no value

		//hide elements marked as optional (enclosed in double square braces
		$content = preg_replace('/\[\[.*_BDT_.*?_BDT_.*\]\]/', '', $content);

		//remove tags from unmatched tags
		$content = preg_replace('/_BDT_.*?_BDT_/', '', $content);

		//remove double square braces from matched optional elements
		$content = str_replace('[[', '', $content);
		$content = str_replace(']]', '', $content);
	}

    return $content;
}

/**
 * To be used in template files to show Replicated tag values.
 * $tag should have the name tag WITHOUT the underscores, for example: <?php replicated_get_value('Name') ?>
 */
function replicated_get_value($tag, $echo = true) {
	$repInfo = unserialize(base64_decode($_COOKIE[REPLICATED_COOKIE]));
	$result = '';

	if($repInfo != '') {
		$tagsArray = get_object_vars($repInfo);

		//Custom fields
		if(startsWith($tag, 'Custom-')) {
			$customField = str_ireplace('Custom-', '', $tag);
			$result = $tagsArray['CustomFields'][$customField];
		}
		else {
			$result = $tagsArray[$tag];
		}
	}

	if($echo) {
		echo $result;
	}
	else {
		return $result;
	}
}

/**
 * Utility method
 */
function startsWith($haystack, $needle)
{
    $length = strlen($needle);
    return (substr($haystack, 0, $length) === $needle);
}

/**
 * Returns rep info if rep info is available, false otherwise
 */
function replicated_is_rep_available() {
	$repInfo = null;

	if(isset($_COOKIE[REPLICATED_COOKIE])) {
		$repInfo = unserialize(base64_decode($_COOKIE[REPLICATED_COOKIE]));
	}

	return $repInfo;

}

/**
 * Handles the after_setup_theme and gets the rep info to
 * store in a cookie for future usage
 */
function replicated_subdomain_handler() {

	if (!is_admin()) {
		$success = '';

		$url = $_SERVER['HTTP_HOST'];
		$host = explode('.', $url);
		$c = count($host);

		if ($c > 2) { // if using a second level domain change this to '3'.
			$repId = $host[0];

			if ($host[0] !== 'www') {

				if (!isset($_COOKIE[REPLICATED_COOKIE])) {
					$success = bd_get_rep_info($repId);
				} else {
					$currentCookieId = $_COOKIE["Current_Rep_ID"];
					if ($currentCookieId !== $repId) {
						$success = bd_get_rep_info($repId);
					}
				}

				if ($success) {
					if (!isset($_COOKIE[REPLICATED_COOKIE])) {
						wp_redirect("https://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'], 302);
						exit;
					} else {
						$currentCookieId = $_COOKIE["Current_Rep_ID"];
						if ($currentCookieId !== $repId) {
							wp_redirect("https://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'], 302);
							exit;
						}
					}
				}

			}
		}
	}

}


/*
 * Gets the Rep info from the Webservice if it does not exist.
 */
function bd_get_rep_info($repId) {
	$success = false;

	setcookie("Current_Rep_ID", $repId, 0 ,'/');

	try {

		$client = new SoapClient(
			API_URL,
			array (
				'soap_version'=>SOAP_1_1,
				'exceptions' => true,
				'trace' => 1,
				'cache_wsdl' => WSDL_CACHE_NONE //WSDL_CACHE_MEMORY,
			)
		);

		$options = get_option('bd_replicated_options');

		$params = array(
			'Credentials' => array('Username'=>$options['bd_replicated_user'],
								 'Password'=>$options['bd_replicated_password'],
								 'Token' => 'foo'),
			'Replicated'  => $repId,
			'UrlReferrer' => get_bloginfo('url')
		);

		$result = $client->GetRepInfo_v3($params);

		if($result->GetRepInfo_v3Result->Success == 1) {

			$repInfo = base64_encode(serialize($result->GetRepInfo_v3Result));

			$success = setcookie(REPLICATED_COOKIE, $repInfo, 0 ,'/');

			if(!$success) {
				$errors = new WP_Error();
				$errors->add('replicated_error', 'Could not store session cookie.');
			}
		}
		else {
			//rep not found
			setcookie(REPLICATED_COOKIE, '', time() - 3600);

			$errors = new WP_Error();
			$errors->add('replicated_error', 'Rep not found.');
		}
	}
	catch (Exception $e) {
		//print_r($e);
		$errors = new WP_Error();
		$errors->add('replicated_error', 'API not available.');
	}

	return $success;
}

/*
 * New function for Contact Form 7 hack.
 */
function replicated_get_value_new($tag) {
	$echo = '';

    $repInfo = unserialize(base64_decode($_COOKIE[REPLICATED_COOKIE]));
    $result = '';

    if($repInfo != '') {
        $tagsArray = get_object_vars($repInfo);

        //Custom fields
        if(startsWith($tag, 'Custom-')) {
            $customField = str_ireplace('Custom-', '', $tag);
            $result = $tagsArray['CustomFields'][$customField];
        }
        else {
            $result = $tagsArray[$tag];
        }
    }

    if($echo) {
        echo $result;
    }
    else {
        return $result;
    }
}


?>